# Generated by Django 3.1.6 on 2021-02-22 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0020_auto_20210125_1007"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventhost",
            name="logo_url",
            field=models.CharField(blank=True, max_length=150),
        ),
    ]



# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color

# AA Contexts
from aadiscordbot.app_settings import get_site_url
from allianceauth.services.modules.discord.models import DiscordUser

# OPCALENDAR
import operator
from opcalendar.models import Event, IngameEvents, EventHost
from django.db.models import Q, F
from itertools import chain
from app_utils.urls import static_file_absolute_url
from datetime import datetime
import os

import logging

from opcalendar.app_settings import (
    OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL,
)

logger = logging.getLogger(__name__)

# i dont want to do this, but the below object get wont work without it, investigate.
os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = "true"


class Ops(commands.Cog):
    """
    A Collection of Authentication Tools for Alliance Auth
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def ops(self, ctx):
        """
        Sends a direct message about the upcoming events visible for the user
        """
        await ctx.trigger_typing()

        # Get authod ID
        id = ctx.message.author.id

        url = get_site_url()

        today = datetime.today()

        user_argument = ctx.message.content[5:]

        if not user_argument:
            host = "all hosts"

        else:
            host = user_argument

        # Get user if discord service is active
        try:
            discord_user = DiscordUser.objects.get(uid=id)

            user = discord_user.user

            discord_active = True

        except Exception:
            logger.error("Discord service is not active for user")

            embed = Embed(title="Command failed")
            embed.colour = Color.red()
            embed.set_thumbnail(
                url=static_file_absolute_url("opcalendar/terminate.png")
            )
            embed.description = "Activate the [discord service]({}/services) to access this command.".format(
                url
            )
            discord_active = False

            await ctx.reply(embed=embed)

        if discord_active:
            # Get normal events
            # Filter by groups and states
            events = (
                Event.objects.filter(
                    Q(event_visibility__restricted_to_group__in=user.groups.all())
                    | Q(event_visibility__restricted_to_group__isnull=True),
                )
                .filter(
                    Q(event_visibility__restricted_to_state=user.profile.state)
                    | Q(event_visibility__restricted_to_state__isnull=True),
                )
                .filter(start_time__gte=today)
            )
            if user_argument:
                events = events.filter(host__community=host)

            # Get ingame events
            # Filter by groups and states
            ingame_events = (
                IngameEvents.objects.annotate(
                    start_time=F("event_start_date"),
                    end_time=F("event_end_date"),
                )
                .filter(
                    Q(
                        owner__event_visibility__restricted_to_group__in=user.groups.all()
                    )
                    | Q(owner__event_visibility__restricted_to_group__isnull=True),
                )
                .filter(
                    Q(owner__event_visibility__restricted_to_state=user.profile.state)
                    | Q(owner__event_visibility__restricted_to_state__isnull=True),
                )
                .filter(start_time__gte=today)
            )

            hosts = EventHost.objects.all()

            if not OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL:
                hosts = hosts.filter(external=False)

            hosts = _hosts(hosts)

            if user_argument:
                ingame_events = ingame_events.filter(host__community=host)

            # Combine events, limit to 20 events
            all_events = sorted(
                chain(events, ingame_events),
                key=operator.attrgetter("start_time"),
            )[:20]

            embed = Embed(title="Scheduled Opcalendar Events")

            embed.set_thumbnail(url=static_file_absolute_url("opcalendar/calendar.png"))

            embed.colour = Color.blue()

            embed.description = "List view of the next 20 upcoming operations for {}. A calendar view is located in [here]({}/opcalendar).\n\nFiltering: To filter events for a specific host add the name after the command ie. `!ops my coalition`\n\nAvailable hosts: *{}*".format(
                host, url, hosts
            )

            # Format all events and ingame events
            for event in all_events:
                if type(event) == Event:
                    embed.add_field(
                        name="Event: {0} {1}".format(
                            event.title, event.operation_type.ticker
                        ),
                        value="Host: {0}\nFC: {1}\nDoctrine: {2}\nLocation: {3}\nTime: {4}\n[Details]({5}/opcalendar/event/{6}/details/)\n".format(
                            event.host,
                            event.fc,
                            event.doctrine,
                            event.formup_system,
                            event.start_time,
                            url,
                            event.id,
                        ),
                        inline=False,
                    )
                if type(event) == IngameEvents:
                    embed.add_field(
                        name="Ingame Event: {0}".format(event.title),
                        value="Host: {0}\n Time:{1}\n[Details]({2}/opcalendar/ingame/event/{3}/details/)".format(
                            event.owner_name,
                            event.start_time,
                            url,
                            event.event_id,
                        ),
                        inline=False,
                    )

            await ctx.author.send(embed=embed)

            embed = Embed(title="Events sent")
            embed.colour = Color.green()
            embed.description = (
                "I have sent you a direct message about upcoming events."
            )
            discord_active = False

            await ctx.reply(embed=embed)


def _hosts(hosts):
    hosts = [x.community for x in hosts]

    if hosts:
        return ", ".join(hosts)
    else:
        return None


def setup(bot):
    bot.add_cog(Ops(bot))



# Generated by Django 3.1.2 on 2020-11-04 12:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0005_auto_20201104_1127"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventhost",
            name="details",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="discord",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="facebook",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="fleet_comms",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="fleet_doctrines",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="ingame_channel",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="ingame_mailing_list",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="twitch",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="website",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="youtube",
            field=models.CharField(blank=True, max_length=150),
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-13 08:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0012_auto_20210112_0934"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventimport",
            name="creator",
            field=models.ForeignKey(
                default="1",
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-25 10:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0019_auto_20210125_1005"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="eventimport",
            options={
                "verbose_name": "NPSI Event Import",
                "verbose_name_plural": "NPSI Event Imports",
            },
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-13 08:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0013_eventimport_creator"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventimport",
            name="creator",
            field=models.ForeignKey(
                default="1",
                help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)",
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="host",
            field=models.ForeignKey(
                default=1,
                help_text="The AA host that will be used for the pulled events",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="operation_type",
            field=models.ForeignKey(
                help_text="Operation type and ticker that will be assigned for the pulled fleets",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventcategory",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="source",
            field=models.CharField(
                choices=[
                    ("Spectre Fleet", "Spectre Fleet"),
                    ("EVE University", "EVE University"),
                ],
                help_text="The API source where you want to pull events from",
                max_length=32,
            ),
        ),
    ]



import requests
import json

from typing import Tuple
from datetime import timedelta, datetime

from django.db import models
from django.urls import reverse
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.html import strip_tags
from django.contrib.auth.models import Group

from esi.errors import TokenExpiredError, TokenInvalidError
from esi.models import Token

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from allianceauth.authentication.models import State

from .providers import esi
from .decorators import fetch_token_for_owner

logger = get_extension_logger(__name__)


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            (
                "basic_access",
                "Can access this app and see operations based on visibility rules",
            ),
            ("create_event", "Can create and edit events"),
            ("see_signups", "Can see all signups for event"),
            ("manage_event", "Can delete and manage other signups"),
            (
                "add_ingame_calendar_owner",
                "Can add ingame calendar feeds for their corporation",
            ),
        )


class WebHook(models.Model):
    """Discord Webhook for pings"""

    name = models.CharField(
        max_length=150,
        help_text=_("Name for this webhook"),
    )
    webhook_url = models.CharField(
        max_length=500,
        help_text=_("Webhook URL"),
    )
    enabled = models.BooleanField(default=True, help_text=_("Is the webhook enabled?"))

    def send_embed(self, embed):
        custom_headers = {"Content-Type": "application/json"}
        data = '{"embeds": [%s]}' % json.dumps(embed)
        r = requests.post(self.webhook_url, headers=custom_headers, data=data)
        r.raise_for_status()

    class Meta:
        verbose_name = "Webhook"
        verbose_name_plural = "Webhooks"

    def __str__(self):
        return "{}".format(self.name)


class EventVisibility(models.Model):
    name = models.CharField(
        max_length=150, null=False, help_text="Name for the visibility filter"
    )
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="eventvisibility_require_groups",
        help_text=_(
            "The group(s) that will be able to see this event visibility type ..."
        ),
    )
    restricted_to_state = models.ManyToManyField(
        State,
        blank=True,
        related_name="eventvisibility_require_states",
        help_text=_(
            "The state(s) that will be able to see this event visibility type ..."
        ),
    )
    webhook = models.ForeignKey(
        WebHook,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Webhook to send over notifications about these fleet types"),
    )
    ignore_past_fleets = models.BooleanField(
        default=True,
        help_text=_("Should we ignore fleet signals that are in the past"),
    )
    color = models.CharField(
        max_length=7,
        default="",
        blank=True,
        help_text=_("Color to be displayed on calendar"),
    )
    include_in_feed = models.BooleanField(
        default=False,
        help_text=("Whether these events should be included in the ical feed."),
    )
    is_visible = models.BooleanField(
        default=True,
        help_text=(
            "Whether this visibility filter should be displayed on the event form. Disable for internal visibilities such as the NPSI import fleet visibilities."
        ),
    )
    is_default = models.BooleanField(
        default=False,
        help_text=(
            "Whether this visibility filter is used as the default value on the event form"
        ),
    )
    is_active = models.BooleanField(
        default=True,
        help_text=("Whether this visibility filter is active"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:
        verbose_name = "Event Visibility Filter"
        verbose_name_plural = "Event Visibilities Filters"

    def save(self, *args, **kwargs):
        if self.is_default:
            # select all other is_default items
            qs = type(self).objects.filter(is_default=True)
            # except self (if self already exists)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            # and deactive them
            qs.update(is_default=False)

        super(EventVisibility, self).save(*args, **kwargs)

    @property
    def get_visibility_class(self):
        return f"{self.name.replace(' ', '-').lower()}"


class EventHost(models.Model):
    """Fleet Timer Create/Delete pings"""

    community = models.CharField(
        max_length=150, null=False, help_text="Name of the community"
    )
    logo_url = models.CharField(
        max_length=256, blank=True, help_text="Absolute URL for the community logo"
    )
    ingame_channel = models.CharField(
        max_length=150, blank=True, help_text="Ingame channel name"
    )
    ingame_mailing_list = models.CharField(
        max_length=150, blank=True, help_text="Ingame mailing list name"
    )
    fleet_comms = models.CharField(
        max_length=150,
        blank=True,
        help_text="Link or description for primary comms such as discord link",
    )
    fleet_doctrines = models.CharField(
        max_length=150, blank=True, help_text="Link or description to the doctrines"
    )
    website = models.CharField(max_length=150, blank=True, help_text="Website link URL")
    discord = models.CharField(max_length=150, blank=True, help_text="Discord link URL")
    twitch = models.CharField(max_length=150, blank=True, help_text="Twitch link URL")
    twitter = models.CharField(max_length=150, blank=True, help_text="Twitter link URL")
    youtube = models.CharField(max_length=150, blank=True, help_text="Youtube link URL")
    facebook = models.CharField(
        max_length=150, blank=True, help_text="Facebook link URL"
    )
    details = models.CharField(
        max_length=150, blank=True, help_text="Short description about the host."
    )
    is_default = models.BooleanField(
        default=False,
        help_text=("Whether this host is used as the default value on the event form"),
    )
    external = models.BooleanField(
        default=False,
        help_text=_(
            "External hosts are for NPSI API imports. Checking this box will hide the host in the manual event form."
        ),
    )

    def __str__(self):
        return str(self.community)

    class Meta:
        verbose_name = "Host"
        verbose_name_plural = "Hosts"

    def save(self, *args, **kwargs):
        if self.is_default:
            # select all other is_default items
            qs = type(self).objects.filter(is_default=True)
            # except self (if self already exists)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            # and deactive them
            qs.update(is_default=False)

        super(EventHost, self).save(*args, **kwargs)


class EventCategory(models.Model):
    name = models.CharField(
        max_length=150,
        help_text=_("Name for the category"),
    )
    ticker = models.CharField(
        max_length=10,
        help_text=_("Ticker for the category"),
    )
    color = models.CharField(
        max_length=7,
        default="",
        blank=True,
        help_text=_("Color to be displayed on calendar"),
    )
    description = models.TextField(
        blank=True,
        help_text="Prefilled description that will be added on default on the event description.",
    )

    class Meta:
        verbose_name = "Category"
        verbose_name_plural = "Categories"

    def __str__(self):
        return str(self.name)

    @property
    def get_category_class(self):
        return f"{self.name.replace(' ', '-').lower()}"


class EventImport(models.Model):
    """NPSI IMPORT OPTIONS"""

    SPECTRE_FLEET = "SF"
    EVE_UNIVERSITY = "EU"
    FUN_INC = "FI"
    FRIDAY_YARRRR = "FY"
    REDEMPTION_ROAD = "RR"
    CAS = "CA"
    FWAMING_DWAGONS = "FD"
    FREE_RANGE_CHIKUNS = "FR"
    EVE_LINKNET = "LN"

    IMPORT_SOURCES = [
        (EVE_LINKNET, _("EVE LinkNet")),
        (SPECTRE_FLEET, _("Spectre Fleet")),
        (EVE_UNIVERSITY, _("EVE University")),
        (FUN_INC, _("Fun Inc.")),
        (FRIDAY_YARRRR, _("FRIDAY YARRRR")),
        (REDEMPTION_ROAD, _("Redemption Road")),
        (CAS, _("CAS")),
        (FWAMING_DWAGONS, _("Fwaming Dwagons")),
        (FREE_RANGE_CHIKUNS, _("FREE RANGE CHIKUNS")),
    ]

    source = models.CharField(
        max_length=32,
        choices=IMPORT_SOURCES,
        help_text="The API source where you want to pull events from",
    )

    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        default=1,
        help_text="The AA host that will be used for the pulled events",
    )
    operation_type = models.ForeignKey(
        EventCategory,
        on_delete=models.CASCADE,
        help_text="Operation type and ticker that will be assigned for the pulled fleets",
    )
    creator = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        default="1",
        help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)",
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character",
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )

    def __str__(self):
        return str(self.source)

    class Meta:
        verbose_name = "NPSI Event Import"
        verbose_name_plural = "NPSI Event Imports"


class Event(models.Model):
    DAILY = "DD"
    WEEKLY = "WE"
    MONTHLY = "MM"
    YEARLY = "YY"

    REPEAT_INTERVAL = [
        (DAILY, _("Daily")),
        (WEEKLY, _("Weekly")),
        (MONTHLY, _("Monthly")),
        (YEARLY, _("Yearly")),
    ]

    operation_type = models.ForeignKey(
        EventCategory,
        null=True,
        on_delete=models.CASCADE,
        help_text=_("Event category type"),
    )
    title = models.CharField(
        max_length=200,
        help_text=_("Title for the event"),
    )
    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        help_text=_("Host entity for the event"),
    )
    doctrine = models.CharField(
        max_length=254,
        help_text=_("Doctrine URL or name"),
    )
    formup_system = models.CharField(
        max_length=254,
        help_text=_("Location for formup"),
    )
    description = models.TextField(
        help_text=_("Description text for the operation"),
    )
    start_time = models.DateTimeField(
        help_text=_("Event start date and time"),
    )
    end_time = models.DateTimeField(
        help_text=_("Event end date and time"),
    )
    repeat_event = models.CharField(
        max_length=32,
        default=False,
        null=True,
        blank=True,
        choices=REPEAT_INTERVAL,
        help_text="Select if you want to repeat this event in the future",
    )
    repeat_times = models.IntegerField(
        default=False,
        null=True,
        blank=True,
        help_text="How many times do you want to repeat this event?",
    )
    fc = models.CharField(
        max_length=254,
        help_text=_("Fleet commander/manager for the event"),
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )
    external = models.BooleanField(
        default=False,
        null=True,
        help_text=_("Is the event an external event over API"),
    )
    created_date = models.DateTimeField(
        default=timezone.now,
        help_text=_("When the event was created"),
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_("Character used to create the event"),
    )
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        help_text=_("User who created the event"),
    )

    def duration(self):
        return self.end_time - self.start_time

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("opcalendar:event-detail", args=(self.id,))

    @property
    def get_visibility_class(self):
        if self.event_visibility:
            return f"{self.event_visibility.name.replace(' ', '-').lower()}"

    @property
    def get_event_styling(self):
        if self.event_visibility:
            return f".{self.event_visibility.name.replace(' ', '-').lower()}:before{{border-color: transparent {self.event_visibility.color} transparent transparent;border-style: solid;}} .{self.operation_type.name.replace(' ', '-').lower()} {{border-left: 6px solid {self.operation_type.color} !important;}}"

    @property
    def get_category_class(self):
        if self.operation_type:
            return f"{self.operation_type.name.replace(' ', '-').lower()}"

    @property
    def get_date_status(self):

        if datetime.now(timezone.utc) > self.start_time:
            return "past-event"
        else:
            return "future-event"

    @property
    def get_html_url(self):
        url = reverse("opcalendar:event-detail", args=(self.id,))
        return f"{url}"

    @property
    def get_html_title(self):
        return f'<span>{self.start_time.strftime("%H:%M")} - {self.end_time.strftime("%H:%M")} <i>{self.host.community}</i></span><span><b>{self.operation_type.ticker} {self.title}</b></span>'

    def user_can_edit(self, user: user) -> bool:
        """Checks if the given user can edit this timer. Returns True or False"""
        return user.has_perm("opcalendar.manage_event") or (
            self.user == user and user.has_perm("opcalendar.create_event")
        )


class Owner(models.Model):
    """A corporation that holds the calendars"""

    ERROR_NONE = 0
    ERROR_TOKEN_INVALID = 1
    ERROR_TOKEN_EXPIRED = 2
    ERROR_INSUFFICIENT_PERMISSIONS = 3
    ERROR_NO_CHARACTER = 4
    ERROR_ESI_UNAVAILABLE = 5
    ERROR_OPERATION_MODE_MISMATCH = 6
    ERROR_UNKNOWN = 99

    ERRORS_LIST = [
        (ERROR_NONE, "No error"),
        (ERROR_TOKEN_INVALID, "Invalid token"),
        (ERROR_TOKEN_EXPIRED, "Expired token"),
        (ERROR_INSUFFICIENT_PERMISSIONS, "Insufficient permissions"),
        (ERROR_NO_CHARACTER, "No character set for fetching data from ESI"),
        (ERROR_ESI_UNAVAILABLE, "ESI API is currently unavailable"),
        (
            ERROR_OPERATION_MODE_MISMATCH,
            "Operaton mode does not match with current setting",
        ),
        (ERROR_UNKNOWN, "Unknown error"),
    ]

    corporation = models.OneToOneField(
        EveCorporationInfo,
        default=None,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        help_text="Corporation owning the calendar",
        related_name="+",
    )
    character = models.ForeignKey(
        CharacterOwnership,
        on_delete=models.SET_DEFAULT,
        default=None,
        null=True,
        blank=True,
        help_text="Character used for syncing the calendar",
        related_name="+",
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )
    operation_type = models.ForeignKey(
        EventCategory,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        help_text=_(
            "Event category that will be assigned for all of the events from this owner."
        ),
    )
    is_active = models.BooleanField(
        default=True,
        help_text=("whether this owner is currently included in the sync process"),
    )

    class Meta:
        verbose_name = "Ingame Clanedar Owner"
        verbose_name_plural = "Ingame Calendar Owners"

    @fetch_token_for_owner(["esi-calendar.read_calendar_events.v1"])
    def update_events_esi(self, token):
        if self.is_active:

            # Get all current imported fleets in database
            event_ids_to_remove = list(
                IngameEvents.objects.filter(owner=self).values_list(
                    "event_id", flat=True
                )
            )
            logger.debug(
                "Ingame events currently in database: %s" % event_ids_to_remove
            )

            events = self._fetch_events()
            for event in events:

                character_id = self.character.character.character_id

                details = (
                    esi.client.Calendar.get_characters_character_id_calendar_event_id(
                        character_id=character_id,
                        event_id=event["event_id"],
                        token=token.valid_access_token(),
                    ).results()
                )

                end_date = event["event_date"] + timedelta(minutes=details["duration"])

                original = IngameEvents.objects.filter(
                    owner=self, event_id=event["event_id"]
                ).first()

                text = strip_tags(details["text"])

                try:
                    if original is not None:

                        logger.debug("Event: %s already in database" % event["title"])
                        event_ids_to_remove.remove(original.event_id)

                    else:

                        # Check if we already have the host
                        original_host = EventHost.objects.filter(
                            community=details["owner_name"]
                        ).first()

                        logger.debug("Got original host: {}".format(original_host))

                        if original_host is not None:
                            host = original_host
                        else:
                            host = EventHost.objects.create(
                                community=details["owner_name"],
                                external=True,
                            )

                        IngameEvents.objects.create(
                            event_id=event["event_id"],
                            owner=self,
                            text=text,
                            event_owner_id=details["owner_id"],
                            owner_type=details["owner_type"],
                            owner_name=details["owner_name"],
                            host=host,
                            importance=details["importance"],
                            duration=details["duration"],
                            event_start_date=event["event_date"],
                            event_end_date=end_date,
                            title=event["title"],
                        )
                        logger.debug("New event created: %s" % event["title"])
                except Exception as e:
                    logger.debug("Error adding new event: %s" % e)

            logger.debug("Removing all events that we did not get over API")
            IngameEvents.objects.filter(pk__in=event_ids_to_remove).delete()

            logger.debug(
                "All events fetched for %s" % self.character.character.character_name
            )

    @fetch_token_for_owner(["esi-calendar.read_calendar_events.v1"])
    def _fetch_events(self, token) -> list:

        character_id = self.character.character.character_id

        events = esi.client.Calendar.get_characters_character_id_calendar(
            character_id=character_id,
            token=token.valid_access_token(),
        ).results()

        return events

    def token(self, scopes=None) -> Tuple[Token, int]:
        """returns a valid Token for the owner"""
        token = None
        error = None

        # abort if character is not configured
        if self.character is None:
            logger.error("%s: No character configured to sync", self)
            error = self.ERROR_NO_CHARACTER

        # abort if character does not have sufficient permissions
        elif self.corporation and not self.character.user.has_perm(
            "opcalendar.add_ingame_calendar_owner"
        ):
            logger.error(
                "%s: This character does not have sufficient permission to sync corporation calendars",
                self,
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        # abort if character does not have sufficient permissions
        elif not self.character.user.has_perm("opcalendar.add_ingame_calendar_owner"):
            logger.error(
                "%s: This character does not have sufficient permission to sync personal calendars",
                self,
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        else:
            try:
                # get token
                token = (
                    Token.objects.filter(
                        user=self.character.user,
                        character_id=self.character.character.character_id,
                    )
                    .require_scopes(scopes)
                    .require_valid()
                    .first()
                )
            except TokenInvalidError:
                logger.error("%s: Invalid token for fetching calendars", self)
                error = self.ERROR_TOKEN_INVALID
            except TokenExpiredError:
                logger.error("%s: Token expired for fetching calendars", self)
                error = self.ERROR_TOKEN_EXPIRED
            else:
                if not token:
                    logger.error("%s: No token found with sufficient scopes", self)
                    error = self.ERROR_TOKEN_INVALID

        return token, error


class IngameEvents(models.Model):

    event_id = models.PositiveBigIntegerField(
        primary_key=True, help_text="The EVE ID of the event"
    )
    owner = models.ForeignKey(
        Owner,
        on_delete=models.CASCADE,
        help_text="Event holder",
    )
    event_start_date = models.DateTimeField()
    event_end_date = models.DateTimeField(blank=True, null=True)
    title = models.CharField(max_length=128)
    text = models.TextField()
    event_owner_id = models.IntegerField(null=True)
    owner_type = models.CharField(max_length=128)
    owner_name = models.CharField(max_length=128)
    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        default=1,
        help_text=_("Host entity for the event"),
    )
    importance = models.CharField(max_length=128)
    duration = models.CharField(max_length=128)

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = "Ingame Event"
        verbose_name_plural = "Ingame Events"

    def get_absolute_url(self):
        return reverse("opcalendar:ingame-event-detail", args=(self.event_id,))

    @property
    def get_date_status(self):

        if datetime.now(timezone.utc) > self.event_start_date:
            return "past-event"
        else:
            return "future-event"

    @property
    def get_visibility_class(self):
        if self.owner.event_visibility:
            return f"{self.owner.event_visibility.name.replace(' ', '-').lower()}"
        else:
            return "ingame-event"

    @property
    def get_event_styling(self):
        d = ""
        if self.owner.event_visibility:
            d += f".{self.owner.event_visibility.name.replace(' ', '-').lower()}:before{{border-color: transparent {self.owner.event_visibility.color} transparent transparent;border-style: solid;}}"
        if self.owner.operation_type:
            d += f".{self.owner.operation_type.name.replace(' ', '-').lower()} {{border-left: 6px solid {self.owner.operation_type.color} !important;}}"
        return d

    @property
    def get_category_class(self):
        if self.owner.operation_type:
            return f"{self.owner.operation_type.name.replace(' ', '-').lower()}"

    @property
    def get_html_url(self):
        url = reverse("opcalendar:ingame-event-detail", args=(self.event_id,))
        return f"{url}"

    @property
    def get_html_title(self):
        return f'<span>{self.event_start_date.strftime("%H:%M")} - {self.event_end_date.strftime("%H:%M")}<i> {self.owner_name}</i></span><span><b>{self.title}</b></span>'


class EventMember(models.Model):
    event = models.ForeignKey(Event, on_delete=models.CASCADE)
    character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character",
    )

    class Meta:
        unique_together = ["event", "character"]



from django.urls import path
from django.conf.urls import url

from . import views

app_name = "opcalendar"
urlpatterns = [
    path("index", views.index, name="index"),
    path("", views.CalendarView.as_view(), name="calendar"),
    path("event/new/", views.create_event, name="event_new"),
    path("add_ingame_calendar/", views.add_ingame_calendar, name="add_ingame_calendar"),
    path("event/edit/<int:event_id>/", views.EventEdit, name="event_edit"),
    path("event/<int:event_id>/details/", views.event_details, name="event-detail"),
    path(
        "ingame/event/<int:event_id>/details/",
        views.ingame_event_details,
        name="ingame-event-detail",
    ),
    path(
        "add_eventmember/<int:event_id>",
        views.EventMemberSignup,
        name="event_member_signup",
    ),
    path("event/<int:event_id>/remove", views.EventDeleteView, name="remove_event"),
    path(
        "remove_eventmember/<int:event_id>",
        views.EventMemberRemove,
        name="event_member_remove",
    ),
    path("feed.ics", views.EventFeed()),
    path(
        "event/<int:event_id>/details/feed.ics",
        views.EventIcalView(),
        name="event-ical-view",
    ),
    url(
        r"^event/new/ajax/get_category/$",
        views.get_category,
        name="get_category",
    ),
]



# Generated by Django 3.1.2 on 2020-11-04 11:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0003_eventhost"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="host",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
    ]



# Generated by Django 3.2.10 on 2022-01-20 11:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0025_auto_20220115_1101'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='repeat_event',
            field=models.CharField(blank=True, choices=[('DD', 'Daily'), ('WE', 'Weekly'), ('MM', 'Monthly'), ('YY', 'Yearly')], default=False, help_text='Select if you want to repeat this event in the future', max_length=32, null=True),
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-04 10:42

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("eveonline", "0012_index_additions"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("view_public", "Can see public events"),
                    ("view_member", "Can see member events"),
                    ("create_event", "Can create and edit events"),
                    ("manage_event", "Can delete and manage signups"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="EventCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=150)),
                ("ticker", models.CharField(max_length=10)),
                (
                    "color",
                    models.CharField(
                        choices=[
                            ("green", "Green"),
                            ("red", "Red"),
                            ("orange", "Orange"),
                            ("blue", "Blue"),
                            ("grey", "Grey"),
                            ("yellow", "Yellow"),
                        ],
                        default="green",
                        max_length=6,
                    ),
                ),
            ],
            options={
                "verbose_name": "Category",
                "verbose_name_plural": "Categories",
            },
        ),
        migrations.CreateModel(
            name="WebHook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=150)),
                ("webhook_url", models.CharField(max_length=500)),
                ("enabled", models.BooleanField()),
            ],
            options={
                "verbose_name": "Webhook",
                "verbose_name_plural": "Webhooks",
            },
        ),
        migrations.CreateModel(
            name="EventSignal",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("ignore_past_fleets", models.BooleanField(default=True)),
                (
                    "webhook",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.webhook",
                    ),
                ),
            ],
            options={
                "verbose_name": "Fleet Signal",
                "verbose_name_plural": "Fleet Signals",
            },
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("doctrine", models.CharField(default="", max_length=254)),
                ("formup_system", models.CharField(default="", max_length=254)),
                ("description", models.TextField()),
                ("start_time", models.DateTimeField()),
                ("end_time", models.DateTimeField()),
                ("fc", models.CharField(default="", max_length=254)),
                (
                    "visibility",
                    models.CharField(
                        choices=[
                            ("Public", "Public access"),
                            ("Member", "Members only access"),
                        ],
                        db_index=True,
                        default="Public",
                        max_length=7,
                    ),
                ),
                (
                    "created_date",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "eve_character",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="eveonline.evecharacter",
                    ),
                ),
                (
                    "operation_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.eventcategory",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EventMember",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.event",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "unique_together": {("event", "user")},
            },
        ),
    ]



# Generated by Django 3.2.10 on 2022-01-15 11:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0024_auto_20210429_1111"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="repeat_event",
            field=models.CharField(
                blank=True,
                choices=[("DD", "Daily"), ("MM", "Monthly"), ("YY", "Yearly")],
                default=False,
                help_text="Select if you want to repeat this event in the future",
                max_length=32,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="repeat_times",
            field=models.IntegerField(
                blank=True,
                default=False,
                help_text="How many times do you want to repeat this event?",
                null=True,
            ),
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-19 17:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0017_auto_20210119_1626"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="ingameevents",
            options={
                "verbose_name": "Ingame Event",
                "verbose_name_plural": "Ingame Events",
            },
        ),
        migrations.AlterModelOptions(
            name="owner",
            options={
                "verbose_name": "Ingame Clanedar Owner",
                "verbose_name_plural": "Ingame Calendar Owners",
            },
        ),
    ]



import socket
from datetime import datetime, timedelta
import random
import string
from typing import Any
from django.contrib import messages
from django.contrib.auth.models import User
from django.conf import settings
from django.contrib.messages.constants import DEBUG, ERROR, INFO, SUCCESS, WARNING
from django.test import TestCase
from django.utils.html import format_html

from esi.models import Scope, Token

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter
from allianceauth.services.hooks import get_extension_logger
from allianceauth.tests.auth_utils import AuthUtils

logger = get_extension_logger(__name__)


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None,
    choices: list = None,
) -> Any:
    """cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overriden)

    `None` allowed as value

    Returns cleaned value for setting
    """
    if default_value is None and not required_type:
        raise ValueError("You must specify a required_type for None defaults")

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        dirty_value = getattr(settings, name)
        if dirty_value is None or (
            isinstance(dirty_value, required_type)
            and (min_value is None or dirty_value >= min_value)
            and (max_value is None or dirty_value <= max_value)
            and (choices is None or dirty_value in choices)
        ):
            cleaned_value = dirty_value
        else:
            logger.warn(
                "You setting for {} it not valid. Please correct it. "
                "Using default for now: {}".format(name, default_value)
            )
            cleaned_value = default_value
    return cleaned_value


class messages_plus:
    """Pendant to Django messages adding level icons and HTML support

    Careful: Use with safe strings only
    """

    _glyph_map = {
        DEBUG: "eye-open",
        INFO: "info-sign",
        SUCCESS: "ok-sign",
        WARNING: "exclamation-sign",
        ERROR: "alert",
    }

    @classmethod
    def _add_messages_icon(cls, level: int, message: str) -> str:
        """Adds an level based icon to standard Django messages"""
        if level not in cls._glyph_map:
            raise ValueError("glyph for level not defined")
        else:
            glyph = cls._glyph_map[level]

        return format_html(
            '<span class="glyphicon glyphicon-{}" '
            'aria-hidden="true"></span>&nbsp;&nbsp;{}',
            glyph,
            message,
        )

    @classmethod
    def debug(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.debug(
            request, cls._add_messages_icon(DEBUG, message), extra_tags, fail_silently
        )

    @classmethod
    def info(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.info(
            request, cls._add_messages_icon(INFO, message), extra_tags, fail_silently
        )

    @classmethod
    def success(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.success(
            request, cls._add_messages_icon(SUCCESS, message), extra_tags, fail_silently
        )

    @classmethod
    def warning(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.warning(
            request, cls._add_messages_icon(WARNING, message), extra_tags, fail_silently
        )

    @classmethod
    def error(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.error(
            request, cls._add_messages_icon(ERROR, message), extra_tags, fail_silently
        )


class SocketAccessError(Exception):
    pass


class NoSocketsTestCase(TestCase):
    """Variation of TestCase class that prevents any use of sockets"""

    @classmethod
    def setUpClass(cls):
        cls.socket_original = socket.socket
        socket.socket = cls.guard
        return super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        socket.socket = cls.socket_original
        return super().tearDownClass()

    @staticmethod
    def guard(*args, **kwargs):
        raise SocketAccessError("Attempted to access network")


class BravadoResponseStub:
    """Stub for IncomingResponse in bravado, e.g. for HTTPError exceptions"""

    def __init__(
        self, status_code, reason="", text="", headers=None, raw_bytes=None
    ) -> None:
        self.reason = reason
        self.status_code = status_code
        self.text = text
        self.headers = headers if headers else dict()
        self.raw_bytes = raw_bytes

    def __str__(self):
        return "{0} {1}".format(self.status_code, self.reason)


class BravadoOperationStub:
    """Stub to simulate the operation object return from bravado via django-esi"""

    class RequestConfig:
        def __init__(self, also_return_response):
            self.also_return_response = also_return_response

    class ResponseStub:
        def __init__(self, headers):
            self.headers = headers

    def __init__(self, data, headers: dict = None, also_return_response: bool = False):
        self._data = data
        self._headers = headers if headers else {"x-pages": 1}
        self.request_config = BravadoOperationStub.RequestConfig(also_return_response)

    def result(self, **kwargs):
        if self.request_config.also_return_response:
            return [self._data, self.ResponseStub(self._headers)]
        else:
            return self._data

    def results(self, **kwargs):
        return self.result(**kwargs)


def add_character_to_user(
    user: User,
    character: EveCharacter,
    is_main: bool = False,
    scopes: list = None,
) -> CharacterOwnership:
    if not scopes:
        scopes = "publicData"

    add_new_token(user, character, scopes)

    if is_main:
        user.profile.main_character = character
        user.profile.save()
        user.save()

    return CharacterOwnership.objects.get(user=user, character=character)


def add_character_to_user_2(
    user: User,
    character_id,
    character_name,
    corporation_id,
    corporation_name,
    alliance_id=None,
    alliance_name=None,
    disconnect_signals=False,
) -> EveCharacter:
    defaults = {
        "character_name": str(character_name),
        "corporation_id": int(corporation_id),
        "corporation_name": str(corporation_name),
    }
    if alliance_id:
        defaults["alliance_id"] = int(alliance_id)
        defaults["alliance_name"] = str(alliance_name)

    if disconnect_signals:
        AuthUtils.disconnect_signals()
    character, _ = EveCharacter.objects.update_or_create(
        character_id=character_id, defaults=defaults
    )
    CharacterOwnership.objects.create(
        character=character, owner_hash=f"{character_id}_{character_name}", user=user
    )
    if disconnect_signals:
        AuthUtils.connect_signals()

    return character


def add_new_token(user: User, character: EveCharacter, scopes: list) -> Token:
    """generates a new Token for the given character and adds it to the user"""
    return _store_as_Token(
        _generate_token(
            character_id=character.character_id,
            character_name=character.character_name,
            scopes=scopes,
        ),
        user,
    )


def _generate_token(
    character_id: int,
    character_name: str,
    access_token: str = "access_token",
    refresh_token: str = "refresh_token",
    scopes: list = None,
    timestamp_dt: object = None,
    expires_in: int = 1200,
) -> dict:

    if timestamp_dt is None:
        timestamp_dt = datetime.utcnow()
    if scopes is None:
        scopes = [
            "esi-mail.read_mail.v1",
            "esi-wallet.read_character_wallet.v1",
            "esi-universe.read_structures.v1",
        ]
    token = {
        "access_token": access_token,
        "token_type": "Bearer",
        "expires_in": expires_in,
        "refresh_token": refresh_token,
        "timestamp": int(timestamp_dt.timestamp()),
        "CharacterID": character_id,
        "CharacterName": character_name,
        "ExpiresOn": _dt_eveformat(timestamp_dt + timedelta(seconds=expires_in)),
        "Scopes": " ".join(list(scopes)),
        "TokenType": "Character",
        "CharacterOwnerHash": _get_random_string(28),
        "IntellectualProperty": "EVE",
    }
    return token


def _dt_eveformat(dt: object) -> str:
    """converts a datetime to a string in eve format
    e.g. '2019-06-25T19:04:44'
    """
    dt2 = datetime(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)
    return dt2.isoformat()


def _get_random_string(char_count):
    return "".join(
        random.choice(string.ascii_uppercase + string.digits) for _ in range(char_count)
    )


def _store_as_Token(token: dict, user: object) -> object:
    """Stores a generated token dict as Token object for given user

    returns Token object
    """
    obj = Token.objects.create(
        access_token=token["access_token"],
        refresh_token=token["refresh_token"],
        user=user,
        character_id=token["CharacterID"],
        character_name=token["CharacterName"],
        token_type=token["TokenType"],
        character_owner_hash=token["CharacterOwnerHash"],
    )
    for scope_name in token["Scopes"].split(" "):
        scope, _ = Scope.objects.get_or_create(name=scope_name)
        obj.scopes.add(scope)

    return obj



from .app_settings import (
    OPCALENDAR_NOTIFY_IMPORTS,
)

from django.dispatch import receiver

from django.db.models.signals import post_save, pre_delete
from .models import Event, IngameEvents
import datetime
from django.utils import timezone

from esi.clients import EsiClientProvider

from .app_settings import get_site_url

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)

RED = 16711710
BLUE = 42751
GREEN = 6684416

esi = EsiClientProvider()


@receiver(post_save, sender=Event)
@receiver(post_save, sender=IngameEvents)
def fleet_saved(sender, instance, created, **kwargs):

    # Ingame Calendar Event
    if sender == IngameEvents and OPCALENDAR_NOTIFY_IMPORTS:
        try:
            logger.debug(
                "New signal created for New Ingame Calendar Event: %s" % instance.title
            )

            url = get_site_url() + "/opcalendar/ingame/event/{}/details/".format(
                instance.pk
            )

            message = "New ingame calendar event: %s" % instance.title

            # Get the entity name from owner name
            entity_id = esi.client.Search.get_search(
                categories=[instance.owner_type],
                search=instance.owner_name,
                strict=True,
            ).results()[instance.owner_type][0]

            logger.debug("Entity data is %s" % entity_id)

            main_char = instance.owner.character.character.character_name

            formup_system = instance.owner_name

            title = instance.title

            eve_time = instance.event_start_date

            fc = instance.owner_name

            # Setup portrait URL based on owner type
            if instance.owner_type == "alliance":

                portrait = "https://images.evetech.net/alliances/%s/logo" % entity_id

                ticker = "[{}]".format(
                    esi.client.Alliance.get_alliances_alliance_id(
                        alliance_id=entity_id
                    ).results()["ticker"]
                )

            if instance.owner_type == "corporation":

                portrait = "https://images.evetech.net/corporations/%s/logo" % entity_id

                ticker = "[{}]".format(
                    esi.client.Corporation.get_corporations_corporation_id(
                        corporation_id=entity_id
                    ).results()["ticker"]
                )

            if instance.owner_type == "character":

                portrait = (
                    "https://images.evetech.net/characters/%s/portrait" % entity_id
                )

                ticker = ""

            logger.debug("Portrait url is %s" % portrait)

            character_name = instance.owner_name

            # If we update instead of delete
            if not created:
                message = "Ingame calendar event updated: %s" % instance.title
                col = BLUE
            else:
                col = GREEN

            embed = {
                "title": message,
                "description": ("%s" % instance.text),
                "url": url,
                "color": col,
                "fields": [
                    {"name": "Owner", "value": fc, "inline": True},
                    {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                    },
                ],
                "footer": {
                    "icon_url": portrait,
                    "text": "{}  {}".format(character_name, ticker),
                },
            }

            hook = instance.owner.event_visibility

            old = datetime.datetime.now(timezone.utc) > eve_time

            if hook.webhook:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        logger.debug("Event is in the past, not sending webhook.")
                    hook.webhook.send_embed(embed)

        except Exception as e:
            logger.error(e)
            pass  # shits fucked... Don't worry about it...

    # For Normal Events
    if sender == Event:
        # For normal events only
        if not instance.external:
            try:
                logger.debug("New signal fleet created for %s" % instance.title)

                url = get_site_url() + "/opcalendar/event/%s/details/" % instance.pk

                title = instance.title

                message = "New event: %s" % title

                main_char = instance.eve_character

                formup_system = instance.formup_system

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "[{}]".format(main_char.corporation_ticker)

                # If we update instead of delete
                if not created:
                    message = "Updated Event: {}".format(title)
                    col = BLUE
                else:
                    col = GREEN

                embed = {
                    "title": message,
                    "description": ("%s" % instance.description),
                    "url": url,
                    "color": col,
                    "fields": [
                        {"name": "FC", "value": fc, "inline": True},
                        {
                            "name": "Type",
                            "value": instance.operation_type.name,
                            "inline": True,
                        },
                        {"name": "Formup", "value": formup_system, "inline": True},
                        {
                            "name": "Eve Time",
                            "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                            "inline": False,
                        },
                    ],
                    "footer": {
                        "icon_url": portrait,
                        "text": " %s %s, %s" % (character_name, ticker, instance.host),
                    },
                }
                hook = instance.event_visibility

                old = datetime.datetime.now(timezone.utc) > eve_time

                if hook.webhook:
                    if hook.webhook.enabled:
                        if old and hook.ignore_past_fleets:
                            logger.debug("Event is in the past, not sending webhook.")
                        hook.webhook.send_embed(embed)

            except Exception as e:
                logger.error(e)
                pass  # shits fucked... Don't worry about it...

        # For automated fleets like NPSI imported fleets. Only post if OPCALENDAR_NOTIFY_IMPORTS set to True
        if instance.external and OPCALENDAR_NOTIFY_IMPORTS:
            try:
                logger.debug("New signal fleet created for %s" % instance.title)

                url = get_site_url() + "/opcalendar/event/%s/details/" % instance.pk

                title = instance.title

                message = "New NPSI event from API: %s" % title

                main_char = instance.eve_character

                formup_system = instance.formup_system

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "[{}]".format(main_char.corporation_ticker)

                # If we update instead of delete
                if not created:
                    message = "Updated Event: {}".format(title)
                    col = BLUE
                else:
                    col = GREEN

                embed = {
                    "title": message,
                    "description": ("%s" % instance.description),
                    "url": url,
                    "color": col,
                    "fields": [
                        {"name": "Community", "value": fc, "inline": True},
                        {
                            "name": "Type",
                            "value": instance.operation_type.name,
                            "inline": True,
                        },
                        {
                            "name": "Eve Time",
                            "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                            "inline": False,
                        },
                    ],
                    "footer": {
                        "icon_url": instance.host.logo_url,
                        "text": " %s" % instance.host,
                    },
                }

                hook = instance.event_visibility

                old = datetime.datetime.now(timezone.utc) > eve_time

                if hook.webhook:
                    if hook.webhook.enabled:
                        if old and hook.ignore_past_fleets:
                            logger.debug("Event is in the past, not sending webhook.")
                        hook.webhook.send_embed(embed)

            except Exception as e:
                logger.error(e)
                pass  # shits fucked... Don't worry about it...


@receiver(pre_delete, sender=Event)
@receiver(pre_delete, sender=IngameEvents)
def fleet_deleted(sender, instance, **kwargs):

    # Ingame Calendar Event
    if sender == IngameEvents and OPCALENDAR_NOTIFY_IMPORTS:
        try:
            logger.debug(
                "New signal created for Deleted Ingame Calendar Event: %s"
                % instance.title
            )

            url = get_site_url() + "/opcalendar/ingame/event/{}/details/".format(
                instance.pk
            )

            message = "Ingame calendar event deleted: %s" % instance.title

            # Get the entity name from owner name
            entity_id = esi.client.Search.get_search(
                categories=[instance.owner_type],
                search=instance.owner_name,
                strict=True,
            ).results()[instance.owner_type][0]

            logger.debug("Entity data is %s" % entity_id)

            main_char = instance.owner.character.character.character_name

            formup_system = instance.owner_name

            title = instance.title

            eve_time = instance.event_start_date

            fc = instance.owner_name

            col = RED

            # Setup portrait URL based on owner type
            if instance.owner_type == "alliance":

                portrait = "https://images.evetech.net/alliances/%s/logo" % entity_id

                ticker = "[{}]".format(
                    esi.client.Alliance.get_alliances_alliance_id(
                        alliance_id=entity_id
                    ).results()["ticker"]
                )

            if instance.owner_type == "corporation":

                portrait = "https://images.evetech.net/corporations/%s/logo" % entity_id

                ticker = "[{}]".format(
                    esi.client.Corporation.get_corporations_corporation_id(
                        corporation_id=entity_id
                    ).results()["ticker"]
                )

            if instance.owner_type == "character":

                portrait = (
                    "https://images.evetech.net/characters/%s/portrait" % entity_id
                )

                ticker = ""

            logger.debug("Portrait url is %s" % portrait)

            character_name = instance.owner_name

            embed = {
                "title": message,
                "description": ("%s" % instance.text),
                "url": url,
                "color": col,
                "fields": [
                    {"name": "Owner", "value": fc, "inline": True},
                    {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                    },
                ],
                "footer": {
                    "icon_url": portrait,
                    "text": "{}  {}".format(character_name, ticker),
                },
            }

            hook = instance.owner.event_visibility

            old = datetime.datetime.now(timezone.utc) > eve_time

            if hook.webhook:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        logger.debug("Event is in the past, not sending webhook.")
                    hook.webhook.send_embed(embed)

        except Exception as e:
            logger.error(e)
            pass  # shits fucked... Don't worry about it...

    # For Normal Events
    if sender == Event:
        # For normal events only
        if not instance.external:
            try:
                logger.debug("New signal fleet created for %s" % instance.title)

                url = get_site_url() + "/opcalendar/event/%s/details/" % instance.pk

                title = instance.title

                message = "Event deleted: %s" % title

                main_char = instance.eve_character

                formup_system = instance.formup_system

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "[{}]".format(main_char.corporation_ticker)

                col = RED

                embed = {
                    "title": message,
                    "description": ("%s" % instance.description),
                    "url": url,
                    "color": col,
                    "fields": [
                        {"name": "FC", "value": fc, "inline": True},
                        {
                            "name": "Type",
                            "value": instance.operation_type.name,
                            "inline": True,
                        },
                        {"name": "Formup", "value": formup_system, "inline": True},
                        {
                            "name": "Eve Time",
                            "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                            "inline": False,
                        },
                    ],
                    "footer": {
                        "icon_url": portrait,
                        "text": " %s %s, %s" % (character_name, ticker, instance.host),
                    },
                }

                hook = instance.event_visibility

                old = datetime.datetime.now(timezone.utc) > eve_time

                if hook.webhook:
                    if hook.webhook.enabled:
                        if old and hook.ignore_past_fleets:
                            logger.debug("Event is in the past, not sending webhook.")
                        hook.webhook.send_embed(embed)

            except Exception as e:
                logger.error(e)
                pass  # shits fucked... Don't worry about it...

        # For automated fleets like NPSI imported fleets. Only post if OPCALENDAR_NOTIFY_IMPORTS set to True
        if instance.external and OPCALENDAR_NOTIFY_IMPORTS:
            try:
                logger.debug("New signal fleet created for %s" % instance.title)

                url = get_site_url() + "/opcalendar/event/%s/details/" % instance.pk

                title = instance.title

                message = "NPSI event deleted from API: %s" % title

                main_char = instance.eve_character

                formup_system = instance.formup_system

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "[{}]".format(main_char.corporation_ticker)

                col = RED

                embed = {
                    "title": message,
                    "description": ("%s" % instance.description),
                    "url": url,
                    "color": col,
                    "fields": [
                        {"name": "Community", "value": fc, "inline": True},
                        {
                            "name": "Eve Time",
                            "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                            "inline": True,
                        },
                    ],
                    "footer": {
                        "icon_url": instance.host.logo_url,
                        "text": " %s" % instance.host,
                    },
                }

                hook = instance.event_visibility

                old = datetime.datetime.now(timezone.utc) > eve_time

                if hook.webhook:
                    if hook.webhook.enabled:
                        if old and hook.ignore_past_fleets:
                            logger.debug("Event is in the past, not sending webhook.")
                        hook.webhook.send_embed(embed)

            except Exception as e:
                logger.error(e)
                pass  # shits fucked... Don't worry about it...



import operator
from calendar import HTMLCalendar
from datetime import date
from itertools import chain
from datetime import datetime
from django.db.models import Q, F
from django.utils import timezone
from allianceauth.services.hooks import get_extension_logger

from .models import Event, IngameEvents
from .app_settings import (
    OPCALENDAR_DISPLAY_STRUCTURETIMERS,
    OPCALENDAR_DISPLAY_MOONMINING,
)

from .app_settings import structuretimers_active, moonmining_active

if structuretimers_active():
    from structuretimers.models import Timer

if moonmining_active():
    from moonmining.models import Extraction

logger = get_extension_logger(__name__)


class Calendar(HTMLCalendar):
    def __init__(self, year=None, month=None, user=None):
        self.year = year
        self.month = month
        self.user = user
        super(Calendar, self).__init__()

    # formats a day as a td
    # filter events by day
    def formatday(
        self, day, events, ingame_events, structuretimer_events, moonmining_events
    ):

        structuretimers_per_day = []

        moonmining_per_day = []

        events_per_day = events.filter(start_time__day=day)

        ingame_events_per_day = ingame_events.filter(event_start_date__day=day)

        if structuretimers_active() and OPCALENDAR_DISPLAY_STRUCTURETIMERS:
            structuretimers_per_day = structuretimer_events.filter(start_time__day=day)

        if moonmining_active() and OPCALENDAR_DISPLAY_MOONMINING:
            moonmining_per_day = moonmining_events.filter(chunk_arrival_at__day=day)

        all_events_per_day = sorted(
            chain(
                events_per_day,
                ingame_events_per_day,
                structuretimers_per_day,
                moonmining_per_day,
            ),
            key=operator.attrgetter("start_time"),
        )

        d = ""

        # Only events for current month
        if day != 0:
            # Parse events
            for event in all_events_per_day:

                if type(event).__name__ == "Timer":
                    OBJECTIVE_UNDEFINED = "UN"
                    OBJECTIVE_HOSTILE = "HO"
                    OBJECTIVE_FRIENDLY = "FR"
                    OBJECTIVE_NEUTRAL = "NE"

                    if event.objective == OBJECTIVE_HOSTILE:
                        objective_verbosed = "Hostile"
                    if event.objective == OBJECTIVE_FRIENDLY:
                        objective_verbosed = "Friendly"
                    if event.objective == OBJECTIVE_NEUTRAL:
                        objective_verbosed = "Neutral"
                    if event.objective == OBJECTIVE_UNDEFINED:
                        objective_verbosed = "Undefined"

                    d += (
                        f'<div class="event {"past-event" if datetime.now(timezone.utc) > event.date else "future-event"} event-structuretimer">'
                        f'<span>{event.date.strftime("%H:%M")}<i> {objective_verbosed} structure timer</i></span>'
                        f"<span>{event.eve_solar_system.name} - {event.structure_type.name}</span>"
                        f"</div>"
                    )

                    logger.debug("Typer type is: %s " % event.get_objective_display())

                if type(event).__name__ == "Extraction":
                    # WIP currently only required view permission
                    if self.user.has_perm(
                        "moonmining.extractions_access"
                    ) and self.user.has_perm("moonmining.extractions_access"):
                        refinery = event.refinery.name
                        system = (
                            event.refinery.moon.eve_moon.eve_planet.eve_solar_system.name
                        )

                        structure = refinery.replace(system, "")

                        d += (
                            f'<a class="nostyling" href="/moonmining/extraction/{event.id}?new_page=yes">'
                            f'<div class="event {"past-event" if datetime.now(timezone.utc) > event.chunk_arrival_at else "future-event"} event-moonmining">'
                            f'<span>{event.chunk_arrival_at.strftime("%H:%M")} <i> Moon chunk arrival {event.refinery.moon.eve_moon.name}</i></span>'
                            f"<span>{structure[3:]}</span>"
                            f"</div>"
                            f"</a>"
                        )
                if (
                    type(event).__name__ == "Event"
                    or type(event).__name__ == "IngameEvents"
                ):
                    d += (
                        f"<style>{event.get_event_styling}</style>"
                        f'<a class="nostyling" href="{event.get_html_url}">'
                        f'<div class="event {event.get_date_status} {event.get_visibility_class} {event.get_category_class}">'
                        f"{event.get_html_title}"
                        f"</div>"
                        f"</a>"
                    )
            if date.today() == date(self.year, self.month, day):
                return f"<td class='today'><div class='date'>{day}</div> {d}</td>"
            return f"<td><div class='date'>{day}</div> {d}</td>"
        return "<td></td>"

    # formats a week as a tr
    def formatweek(
        self, theweek, events, ingame_events, structuretimer_events, moonmining_events
    ):
        week = ""
        for d, weekday in theweek:
            week += self.formatday(
                d, events, ingame_events, structuretimer_events, moonmining_events
            )
        return f"<tr> {week} </tr>"

    # formats a month as a table
    # filter events by year and month

    def formatmonth(self, withyear=True):
        # Get normal events
        # Filter by groups and states
        events = (
            Event.objects.filter(
                start_time__year=self.year,
                start_time__month=self.month,
            )
            .filter(
                Q(event_visibility__restricted_to_group__in=self.user.groups.all())
                | Q(event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(event_visibility__restricted_to_state=self.user.profile.state)
                | Q(event_visibility__restricted_to_state__isnull=True),
            )
        )
        # Get ingame events
        # Filter by groups and states
        ingame_events = (
            IngameEvents.objects.filter(
                event_start_date__year=self.year, event_start_date__month=self.month
            )
            .annotate(start_time=F("event_start_date"), end_time=F("event_end_date"))
            .filter(
                Q(
                    owner__event_visibility__restricted_to_group__in=self.user.groups.all()
                )
                | Q(owner__event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(owner__event_visibility__restricted_to_state=self.user.profile.state)
                | Q(owner__event_visibility__restricted_to_state__isnull=True),
            )
        )

        # Check if structuretimers is active
        # Should we fetch timers

        if structuretimers_active() and OPCALENDAR_DISPLAY_STRUCTURETIMERS:
            structuretimer_events = (
                Timer.objects.all()
                .visible_to_user(self.user)
                .annotate(start_time=F("date"))
                .filter(date__year=self.year, date__month=self.month)
            )
        else:
            structuretimer_events = Event.objects.none()

        # Check if moonmining is active
        # Should we fetch extractions
        if moonmining_active() and OPCALENDAR_DISPLAY_MOONMINING:
            moonmining_events = (
                Extraction.objects.all()
                .annotate(start_time=F("chunk_arrival_at"))
                .filter(
                    chunk_arrival_at__year=self.year, chunk_arrival_at__month=self.month
                )
                .exclude(status="CN")
            )
        else:
            moonmining_events = Event.objects.none()

        logger.debug(
            "Returning %s extractions, display setting is %s. List is: %s"
            % (
                moonmining_events.count(),
                OPCALENDAR_DISPLAY_MOONMINING,
                moonmining_events,
            )
        )

        logger.debug("Returning %s ingame events" % ingame_events.count())

        cal = '<table class="calendar">\n'
        cal += f"{self.formatmonthname(self.year, self.month, withyear=withyear)}\n"
        cal += f"{self.formatweekheader()}\n"

        for week in self.monthdays2calendar(self.year, self.month):
            cal += f"{self.formatweek(week, events, ingame_events, structuretimer_events, moonmining_events)}\n"

        cal += "</table>"

        return cal



# Generated by Django 3.1.6 on 2021-02-22 12:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0021_eventhost_logo_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventhost",
            name="community",
            field=models.CharField(help_text="Name of the community", max_length=150),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="details",
            field=models.CharField(
                blank=True,
                help_text="Short description about the host.",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="discord",
            field=models.CharField(
                blank=True, help_text="Discord link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="facebook",
            field=models.CharField(
                blank=True, help_text="Facebook link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="fleet_comms",
            field=models.CharField(
                blank=True,
                help_text="Link or description for primary comms such as discord link",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="fleet_doctrines",
            field=models.CharField(
                blank=True,
                help_text="Link or description to the doctrines",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="ingame_channel",
            field=models.CharField(
                blank=True, help_text="Ingame channel name", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="logo_url",
            field=models.CharField(
                blank=True,
                help_text="Absolute URL for the community logo",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="twitch",
            field=models.CharField(
                blank=True, help_text="Twitch link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="twitter",
            field=models.CharField(
                blank=True, help_text="Twitter link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="youtube",
            field=models.CharField(
                blank=True, help_text="Youtube link URL", max_length=150
            ),
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-05 08:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0007_eventhost_twitter"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="event",
            unique_together={("title", "start_time")},
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-19 16:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0012_index_additions"),
        ("authentication", "0017_remove_fleetup_permission"),
        ("opcalendar", "0016_auto_20210119_1545"),
    ]

    operations = [
        migrations.AlterField(
            model_name="owner",
            name="character",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Character used for syncing the calendar",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="authentication.characterownership",
            ),
        ),
        migrations.AlterField(
            model_name="owner",
            name="corporation",
            field=models.OneToOneField(
                blank=True,
                default=None,
                help_text="Corporation owning the calendar",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
    ]
